import fedmsg
import fedmsg.encoding
import fedmsg.consumers

from .jobtrigger import JobTrigger
from . import exceptions as exc

ITEM_TYPE = 'compose'


class ComposeCompletedJobTrigger(JobTrigger):
    def process(self, msg):
        topic = msg['topic']

        if not 'branched.complete' in topic and not 'rawhide.complete' in topic:
            raise exc.TriggerMsgError('rejecting message not being branched/rawhide '
                                      'complete compose')

        branch = msg['msg']['branch']
        arch = msg['msg']['arch']

        # non empty arch means secondary arch
        if arch:
            raise exc.TriggerMsgError('rejecting message not being primary arch: %s' % arch)

        self.log.debug('should be scheduling job for completed compose: %s' % branch)

        data = {
            "_msg": msg,
            "message_type": "ComposeCompleted",
            "item": branch,
            "item_type": ITEM_TYPE,
            "branch": branch,
        }
        return data


class ComposeCompletedJobConsumer(fedmsg.consumers.FedmsgConsumer):
    topic = "org.fedoraproject.prod.compose.*"
    jsonify = False
    config_key = 'taskotron.composecompletedjobconsumer.enabled'

    def __init__(self, *args, **kw):
        super(ComposeCompletedJobConsumer, self).__init__(*args, **kw)
        self.trigger = ComposeCompletedJobTrigger(self.log)

    def consume(self, message):
        msg = fedmsg.encoding.loads(message.body)
        try:
            data = self.trigger.process(msg)
            self.trigger.do_trigger(data)
        except exc.TriggerMsgError, e:
            self.log.debug(e)
            return
