import argparse
import logging

import requests
from . import config
from . import exceptions as exc

from .koji_build_msg import KojiBuildCompletedJobTrigger
from .dist_git_commit_msg import DistGitCommitJobTrigger
from .compose_complete_msg import ComposeCompletedJobTrigger


# set up logging
logger = logging.getLogger('Jobrunner')
sh = logging.StreamHandler()
formatter = logging.Formatter(fmt='%(asctime)s - %(name)s - %(levelname)s: %(message)s',
                              datefmt='%Y-%m-%d %H:%M:%S')
sh.setFormatter(formatter)
logger.addHandler(sh)


def datagrepper_jobs(start, end):
    topics = [{
        'start': start,
        'end': end,
        'category': 'buildsys',
        'topic': 'org.fedoraproject.prod.buildsys.build.state.change',
        'page': 1,
    }, {
        'start': start,
        'end': end,
        'category': 'compose',
        'topic': 'org.fedoraproject.prod.compose.*',
        'page': 1,
    }, {
        'start': start,
        'end': end,
        'category': 'git',
        'topic': 'org.fedoraproject.prod.git.receive',
        'page': 1,
    }]

    for params in topics:
        while True:
            r = requests.get(config.datagrepper_url, params=params)
            pages = r.json()['pages']
            page = r.json()['arguments']['page']

            if not pages:
                break

            logger.info('Processing jobs from datagrepper for topic '
                        '%s: page %d of %d', params['topic'], page, pages)

            for msg in r.json()['raw_messages']:
                if msg['topic'].endswith('build.state.change'):
                    trigger = KojiBuildCompletedJobTrigger(logger)
                elif 'compose' in msg['topic']:
                    trigger = ComposeCompletedJobTrigger(logger)
                elif msg['topic'].endswith('git.receive'):
                    trigger = DistGitCommitJobTrigger(logger)
                else:
                    logger.warning('Got an unexpected topic %s', msg['topic'])
                    continue

                try:
                    data = trigger.process(msg)
                    trigger.do_trigger(data)
                except exc.TriggerMsgError, e:
                    logger.debug(e)
                except KeyError, e:
                    logger.exception(e)
                    logger.info('Missing a key in the fedmsg. Maybe consumer needs an update or '
                                'you\'re replaying fedmsgs that have old format?')

            next_page = page + 1
            if next_page > pages:
                break
            params['page'] = next_page


def main():
    parser = argparse.ArgumentParser(prog='Jobrunner',
                                     usage='Gets jobs from datagrepper from a given time '
                                           'period and runs them.')
    parser.add_argument('-s', '--start', help='Start time (Unix time format)')
    parser.add_argument('-e', '--end', help='End time (Unix time format)')
    parser.add_argument('-d', '--debug', action='store_true', help='Print debug message')
    args = parser.parse_args()

    logger.setLevel(logging.DEBUG if args.debug else logging.INFO)

    datagrepper_jobs(args.start, args.end)
